#!/usr/local/bin/perl

use strict;
use TIGR::Foundation;
use File::Basename;

my $base = new TIGR::Foundation;

if (! defined $base){
    die("Cannot create foundation object\n");
}

my $VERSION = '$Revision';

$base->setVersionInfo($VERSION);

my $output;
my $myfasta;
my $evidence_file;
my $treedir;
my $idir_list;
my $addPrefix;
my $useLinComb;

my $HELP_INFO = q~
    run_jigsaw.pl [options]

    Input Options:
       -f <fasta file> - name of fasta formatted genomic sequence
       -o <jigsaw output file name> - name of file for gene predictions
       -e <evidence_file>    - template of evidence file, should be same one used for training
       -d <output directory>    - directory to read decision trees from
       -l <input directories>    - the assumption is, a single directory contains
                                   all of the information for one genomic sequence
                                   this is a file containing a list of those directories (1 per line)
    Miscellaneous Options:
       -h, -help        - print this help message
       -V               - obtain program version
    ~;

$base->setHelpInfo($HELP_INFO);

my $err = $base->TIGR_GetOptions(
				 "e=s" => \$evidence_file,
				 "o=s" => \$output,
				 "d=s" => \$treedir,
				 "l=s" => \$idir_list,
				 "f=s" => \$myfasta,
				 "x" => \$addPrefix,
				 "lin" => \$useLinComb
				 );

if ( defined $addPrefix ) {
  $addPrefix = 0;
} else {
  $addPrefix = 1;
}

if ( ! defined $myfasta ) {
    $base->bail("You must specify a fasta file using -f ");
}

if ( ! defined $output ) {
    $base->bail("You must specify an file name for jigsaw output using -o ");
}

if ( ! defined $idir_list ) {
    $base->bail("You must specify file listing directory names using -l ");
}

if ( ! defined $evidence_file ) {
    $base->bail("You must specify an evidence file using -e ");
}

if ( ! defined $treedir && ! defined $useLinComb ) {
    $base->bail("You must specify a directory to place the decision trees using -d");
}

#runjigsaw($idir_list,$treedir,$evidence_file,$addPrefix,$useLinComb);

#sub runjigsaw {
  #my ($asmblfile,$treedir,$evidence_file,$addPrefix,$useLinComb) = @_;
  my $evidence;
  open(FILE,$evidence_file) || $base->bail("Cannot open $evidence_file: $!");
  while(my $line = <FILE>) {
    chomp($line);
    ## ignore the "answers line, not used when actually running jigsaw
    if( $line !~ / curation/ ) {
      $evidence .= "$line:";
    }
  }
  close(FILE);

  my @testOn;
  open(FILE,$idir_list) || die "unable to open [$idir_list]\n";
  while(my $line = <FILE>) {
    chomp($line);
    push(@testOn,$line);
  }
  close(FILE);
  my $combid = basename($evidence_file);
  for(my $cnt = 0; $cnt <= $#testOn; $cnt++) {
    my $dir = $testOn[$cnt];
    my $dname = basename($dir);
    my $prefix;
    if( $addPrefix ) {
      $prefix = "$dir/$dname.";
    } else {
      $prefix = "$dir/";
    }
    my $combd = "${prefix}${combid}";
    writeEvidenceFile($combd,$evidence,$prefix);
    my $combout = "${prefix}$output";
    my $cmd = "jigsaw -f ${prefix}$myfasta -d $treedir -e $combd -m $combout";
    print "[$cmd]\n";
    system($cmd);
  }
#}

sub writeEvidenceFile {
  my ($tfname,$evlst,$prefix) = @_;
  my @evlst = split(/:/,$evlst);
  my $evdata = "$tfname";
  open(EVDATA,">$evdata") || die "unable to write [$evdata]\n";
  for(my $iter = 0; $iter <= $#evlst; $iter++) {
    print EVDATA "$prefix$evlst[$iter]\n";
  }
  close(EVDATA);
}
