//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, The Institute for Genomic Research (TIGR), Rockville,
// Maryland, U.S.A.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////
#include "Main.h"
#include "Options.h"
#include "InputData.h"
#include "Combine.h"
#include "dsu.h" // my dna string utilities (starting over)
#include "CustomScore.h"
#include <stdexcept>
#include <string>
#include <fstream>
#include <strstream>
#include <time.h>
#include <stdlib.h>

using std::ofstream;
using std::cout;
using std::cerr;
using std::endl;
using std::invalid_argument;

Main::Main(const Options& options, CustomScore* scoringSystem, InputData& inputData, 
	   const string& theSequence, const string& sequenceHeader, const string& program_id) {
  srand( time(NULL));
  if(options.quit) return;
  try {
    if(!options.doTraining()) {
      ofstream combinerOutputStream(options._outputGeneModelFile.c_str()); 
      if(!combinerOutputStream) { 
	cerr<<"combiner output genemodels file, unable to open: ["<<options._outputGeneModelFile.c_str()<<"]"<<endl; 
	throw invalid_argument("could not continue");
      }
      const bool forceWholeGeneModel = options._forceModels;
      string strid = string(&sequenceHeader.c_str()[1]) + string("\t") + program_id;
      mgenes(inputData,theSequence,combinerOutputStream,forceWholeGeneModel,scoringSystem,options,strid);
    } else {
      Combine comb(inputData,theSequence,true,*scoringSystem, options._cutOff);
      const string& matrixFileName = options._matrixFile;
      if(matrixFileName.size() > 0) {
	ofstream ofs(matrixFileName.c_str());
	const list<RegionPrediction*>& rpall = comb.getOutput();
	list<RegionPrediction*>::const_iterator iter = rpall.begin();
	const list<RegionPrediction*>::const_iterator stop = rpall.end();
	for(; iter != stop; ++iter) {
	  //ofs<<**iter<<endl;
	  (*iter)->print(ofs,*scoringSystem);
	  ofs<<endl;
	}
	//ofs<<"Annotation Matrix: "<<endl<<anot<<endl;
      }
    }
  } catch(const invalid_argument& ia) {
    cerr<<ia.what()<<endl;
  }
}

void Main::mgenes(const InputData& inputData, const string& theSequence, std::ostream& outStream, 
		  bool forceWholeGeneModel, CustomScore* cs, const Options& options, const string& strid) {
  const string& matrixFileName = options._matrixFile;
  int cutOff = options._cutOff;
  int modelIdx = 1;
  Combine comb(inputData,theSequence,forceWholeGeneModel,*cs, cutOff);
  if(!comb.empty()) {
    Annotation& anot = comb.getAnnotation();
    anot.printOptimalParse(outStream,modelIdx,*cs,strid);
    if(matrixFileName.size() > 0) {
      ofstream ofs(matrixFileName.c_str());
      ofs<<"Annotation Matrix: "<<endl;
      anot.print(ofs,*cs);
      ofs<<endl;
    }
  } else {
    cout<<"no data!!\n";
  }
}
