//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, The Institute for Genomic Research (TIGR), Rockville,
// Maryland, U.S.A.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////
#include "GenePredDataSet.h"
#include "GenePrediction.h"
#include "Options.h"
#include "Terminal.h"
#include "Initial.h"
#include "Single.h"
#include "Internal.h"
#include "Intron.h"
#include <stdexcept>
#include <assert.h>
#include <list>

using std::cout;
using std::endl;
using std::runtime_error;
using std::list;

GenePredDataSet::GenePredDataSet(const SeqLoc& sl, int geneId, const string& str) : 
   DataSetAbstractProduct(sl, geneId, str,AnnotationScore::defaultScore()) {
}

void GenePredDataSet::print(std::ostream& os, const string& str) const {
  Prediction::print(os,str);
  os<<endl;
  DataStorageType::const_iterator iter = begin();
  while( iter != end() ) {
    const SeqLoc& sl = **iter;
    sl.print(os,str+"\t");
    os<<endl;
    ++iter;
  }
}

void GenePredDataSet::insert(const string& source, int geneId, const SeqLoc& theSl, const string& type, double score) {
  const Options& options = *(Options::getInstance());
  // THIS IS AN OPTIMIZATION which goes a bug for the intron calculation during training 
  // if exons are thrown away, the intron doesn't get properly represented needs to be
  // fixed before you can restore this.
#if 0
  if( (options._subEnd5 != -1 && options._subEnd3 != -1) &&
      (theSl.getEnd3() < options._subEnd5 || theSl.getEnd5() > options._subEnd3)) {
    return;
  }
#endif
  DataStorageType::iterator iter = begin();
  Prediction* currLoc = NULL;
  if(options.isLinComb())
    score = 1.0; // prediction scores ignored.
  while(iter != end()) {
    Prediction& sl = **iter;
    // THIS getModelId is a hack and should
    // be removed at your earliest conveniance!
    // Only in gene prediction data is there
    // relevance of a sequential series of models
    // and the model the "prevPredModel*" stuff
    // should handle this approriately,
    assert(sl.getModelId() != -1);
    if(geneId == sl.getModelId()) {
      currLoc = &sl;
      break;
    }
    ++iter;
  }
  bool newGene = false;
  if(!currLoc) {
    currLoc = new GenePredDataSet(theSl,geneId,source);
    newGene = true;
  }

  DataSetAbstractProduct& currLocGene = dynamic_cast<DataSetAbstractProduct&>(*currLoc);
  Prediction *thePrediction = NULL;

  const GeneModelType* exon_type = NULL;
  if(type == "Internal" || type == "internal") {
    exon_type = &(Internal::getInstance());
  } else if(type == "Terminal" || type == "terminal")
    exon_type = &(Terminal::getInstance());
  else if(type == "Initial" || type == "initial")
    exon_type = &(Initial::getInstance());
  else if(type == "Single" || type == "single")
    exon_type = &(Single::getInstance());
  else {
    throw runtime_error("unrecognized exon type: "+type);
  }
  thePrediction = new GenePrediction(theSl,score,-1,"",*exon_type);
  assert( thePrediction );
  currLocGene.insert( thePrediction );
  if(newGene)
    DataSetAbstractProduct::insert(currLoc);
}

void
GenePredDataSet::add_introns_help() {
  const DataStorageType::const_iterator stop = end();
  DataStorageType::const_iterator iter1 = begin();
  const dsu::Strand_t strnd = getStrnd();
  assert(strnd != dsu::eEither);
  list<Prediction*> tlst;
  while(iter1 != stop) {
    GenePrediction* model1 = dynamic_cast<GenePrediction*>(*iter1);
    DataStorageType::const_iterator iter2 = iter1;
    ++iter2;
    while(iter2 != stop) {
      GenePrediction* model2 = dynamic_cast<GenePrediction*>(*iter2);
      if( !isOverlap(*model1,*model2) ) {
	assert(model2->getScoreIdx() != Intron::getInstance().getScoreIdx());
	const int rghtSide = model1->getEnd3()+1;
	const int leftSide = model2->getEnd5()-1;
	SeqLoc nl(strnd,rghtSide,leftSide);
	const GeneModelType& exon_type = Intron::getInstance();
	const double score = (model1->getScore().getVal()+model2->getScore().getVal())/2.0;
	Prediction* thePrediction = new GenePrediction(nl,score,-1,"",exon_type);
	tlst.push_back(thePrediction);
	iter2 = stop;
      } else {
	iter1 = iter2;
	break;
      }
    }
    if(iter1 != iter2) {
      ++iter1;
    }
  }
  list<Prediction*>::iterator iter = tlst.begin();
  const list<Prediction*>::iterator tstop = tlst.end();
  for(; iter != tstop; ++iter) {
    DataSetAbstractProduct::insert(*iter);
  }
}

static const string geneStr = "One_Gene_Prediction_Set";
const string& GenePredDataSet::getTypeStr() const {
  return geneStr;
}
