//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, The Institute for Genomic Research (TIGR), Rockville,
// Maryland, U.S.A.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////
#ifndef HOMOLOGYDATASET_H
#define HOMOLOGYDATASET_H

#include "DataSetAbstractProduct.h"
class MatchPrediction;

/**
 * <b>Responsibilities:</b><br>
 * 
 * Contains all the Gene Prediction data from a sequence similarity data source.<BR>
 * Specialized version of DataSetAbstractProduct which specifies the basic format that all input<BR>
 * data must adhere to.<BR>
 * I originally was goinging to have a separate representation for distinc types of homology data
 * like "HOMOLOGY", but this isn't needed right now so the Homology namesake got used accross the board
 * plan to change this name to HomologyDataSet
 * 
 * <b>Collaborators:</b><br>
 * HomologyDataSetConcreteFactory, DataSetAbstractProduct<br>
 */
class HomologyDataSet : public DataSetAbstractProduct {
  friend class HomologyDataSetConcreteFactory;
  friend class BtabFactory;
  friend class GffFactory;
public:
  HomologyDataSet(const SeqLoc&, const string&);
  void print(std::ostream& os, const string&) const;

  const string& getTypeStr() const;

private:
  void add_introns_help();
  void insert(MatchPrediction*,const string&);

};

#endif //HOMOLOGYDATASET_H
