#include "ParamFile.h"
#include <fstream>
#include <stdexcept>
#include <string>
#include <strstream>
#include <iostream>

using std::string;
using std::endl;
using std::cout;
using std::ifstream;
using std::ofstream;
using std::ostrstream;
using std::istrstream;
using std::runtime_error;


ParamFile* ParamFile::_instance = NULL;

ParamFile::ParamFile(const string& file) {
  load(file);
}

void
ParamFile::createDefault(const string& file) {
  ofstream ofs(file.c_str());
  if( !ofs ) {
    ostrstream emsg;
    emsg<<"Unable to create default parameter file ["<<file<<"]"<<endl;
    throw runtime_error(emsg.str());
  }
  ofs<<"# Intron Length Penalty"<<endl;
  ofs<<"5000 10"<<endl;
  ofs<<"# Intergenic Length Penalty"<<endl;
  ofs<<"0 0"<<endl;
  ofs<<"# Internal Exon Length Penalty"<<endl;
  ofs<<"0 0"<<endl;
  ofs<<"# Minimum Intron Length"<<endl;
  ofs<<"10"<<endl;
  ofs<<"# Minimum Intergenic Length"<<endl;
  ofs<<"20"<<endl;
  ofs<<"# Alignment Connection Cutoff"<<endl;
  ofs<<"2500"<<endl;
}

void
ParamFile::load(const string& file) {
  ifstream ifs(file.c_str());
  if( !ifs ) {
    //ostrstream emsg;
    cout<<"Unable to open parameter file["<<file<<"]"<<endl;
    cout<<"Creating default file: ["<<file<<"]"<<endl;
    //throw runtime_error(emsg.str());
    createDefault(file);
  }
  const int buff_size = 4096;
  char buff[buff_size];
  char buff2[buff_size];
  while( ifs.getline(buff,buff_size) ) {
    ifs.getline(buff2,buff_size);
    istrstream istrm(buff2);
    if( strstr(buff,"Intron Length Penalty") ) {
      istrm>>_intronLenPenalty>>_intronPenalty;
    } else if( strstr(buff,"Intergenic Length Penalty") ) {
      istrm>>_igrLenPenalty>>_igrPenalty;
    } else if( strstr(buff,"Internal Exon Length Penalty") ) {
      istrm>>_internalExonLenPenalty>>_internalExonPenalty;
    } else if( strstr(buff,"Minimum Intron Length") ) {
      istrm>>_minIntronLen;
    } else if( strstr(buff,"Minimum Intergenic Length") ) {
      istrm>>_minIgrLen;
    } else if( strstr(buff,"Alignment Connection Cutoff") ) {
      istrm>>_minAlignmentConnectionCutoff;
    } else {
      string err = string("Unable to parse string: ") + buff;
      throw runtime_error(err.c_str());
    }
  }
}
