//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, The Institute for Genomic Research (TIGR), Rockville,
// Maryland, U.S.A.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////
#include "Main.h"
#include "Compare.h"
#include "ExonDistr.h"
#include "PrintVec.h"
#include "Custom7.h"
#include "InputData.h"
#include <fstream>
#include <stdexcept>
#include <strstream>

using std::cerr;
using std::endl;
using std::cout;

static void usage(const string&);

int main(int argc, char** argv) 
{
  try 
    {
      string cvs_tag = "3.1.1a";
      string id = string(argv[0]) + string("-") + cvs_tag;
      string cid = id + string(": predict gene models from multiple sources of evidence.");
      Options::createInstance(argc,argv,cid);
      Options& options = *(Options::getInstance());
      options._lnCmb = true; // global variable saying this is the linear combiner
      ExonDistr::createInstance(options._exonDistrFile);
      if( (options.m_fastaFile.size()==0 || 
	   options.m_evidenceListFile.size()==0)  && 
	  options._vecFile.size()==0 ) {
	usage(argv[0]);
	return 0;
      }
      if(options.quit) return 0;

      const char* procfile = options.m_evidenceListFile.c_str();
      string theSequence, sequenceHeader;
      dsu::readFastaFile(theSequence,options.m_fastaFile);
      dsu::readFastaHeader(sequenceHeader,options.m_fastaFile);
      CustomScore* cs = NULL;
      Custom7::newInstance();
      cs = Custom7::getInstance(); 
      InputData* inputData = new InputData(procfile,theSequence,cs->getFeatMaps());
      Main(options, cs, *inputData, theSequence,sequenceHeader,id);
      delete cs;
      delete inputData;			
    } 
  catch(const std::exception& err) 
    {
      cerr<<err.what()<<endl;
      cerr<<"Unable to continue."<<endl;
    }
  return 0;
}

void usage(const string& str) {
    cout<<"----------------------------------------------------------------"<<endl;
	 cout<<"Combiner: predict gene models from multiple sources of evidence."<<endl;
    cout<<"----------------------------------------------------------------"<<endl<<endl;
    cout<<"usage: "<<str<<" -f \"fasta file\" -e \"evidence file\" -m \"output gene models\" [-h] [-x] [-t filename] [-p \"end5 end3\"] [-v] [-V] [-i \"minimum intron length (number of nucleotides)\"]\n";
    cout<<"\t-f\tCombiner will construct gene models for the genomic sequence specified in this file\n";
    cout<<"\t-e\tfile lists the location of each evidence file and its associated type.\n";
    cout<<"\t\tformat is: filename (locatable path), file format (default or btab), evidence type (homology, geneprediction or spliceprediction),\n";
    cout<<"\t\t\tweight (homology,geneprediction only)\n";
    cout<<"\t\texample:\n";
    cout<<"\t\t/data/estAlignments.ph default homology 0.2\n";
    cout<<"\t\t/data/glimmerM.gp default geneprediction 0.49\n";
    cout<<"\t\t/data/genesplicer.sp default spliceprediction\n\n";
    cout<<"\t-m\tname of output file to place predicted genemodel list in\n";
    cout<<"\t-i\tspecify the minimum length of introns (number of nucleotides)\n";
    cout<<"\t-x\tname of output file to place matrix info in (optional)\n";
    cout<<"\t-p\tsubset of a specified sequence\n";
    cout<<"\t-c\tspecify the number of base pairs between coding regions from which to stop trying to make connections (optional)\n";
    cout<<"\t\twhen no number provided, all possible combinations are considered. Becuase this can take more time, this cutoff\n";
    cout<<"\t\toption is provided, for conveniance\n";
    cout<<"\t-h\tprints out this usage message\n";
    cout<<"\t-V\tVerbose output\n";
    cout<<"\t-v\tversion information\n";
    cout<<"\n\tA typical execution of the combiner from the command line:\n";
    cout<<"\t"<<str<<" -f sequence.fasta -e evidence.data -m outputmodel.gp\n\n";
}
